import platform

def get_os_type():
    os_type = platform.system()
    if os_type == "Darwin":
        return "mac"
    elif os_type == "Windows":
        return "windows"
    elif os_type == 'Linux':
        return "linux"


import logging
import requests
from requests import Session
import json
from bs4 import BeautifulSoup
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from abc import ABCMeta, abstractmethod
from catscore.http.request import CatsRequest
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

class CatsWebDriver:
    def __init__(self, binary_location, executable_path, proxy, headless):
        """[summary]

        Arguments:
            binary_location {[type]} -- [description]
            executable_path {[type]} -- [description]
            proxy {[type]} -- [description]
            headless {[type]} -- [description]
        """
        options = Options()
        options.binary_location = binary_location
        logging.info(f"WebDriverSession.__init__ : {binary_location}, {executable_path}, {proxy}, {headless}")
        if headless:
            options.add_argument('--headless')
            # https://www.ytyng.com/blog/ubuntu-chromedriver/
            options.add_argument("--disable-dev-shm-usage") # overcome limited resource problems
            options.add_argument("start-maximized") # open Browser in maximized mode
            options.add_argument("disable-infobars") # disabling infobars
            options.add_argument("--disable-extensions") # disabling extensions
            options.add_argument("--disable-gpu") # applicable to windows os only
            options.add_argument("--no-sandbox") # Bypass OS security model
        if proxy:
            logging.info("WebDriverSession proxy on")
            options.add_argument(f"proxy-server={proxy}")

        caps = DesiredCapabilities.CHROME
        caps['loggingPrefs'] = {'performance': 'INFO'}
        self.driver = webdriver.Chrome(options=options, executable_path=executable_path,desired_capabilities=caps)
        self.driver.implicitly_wait(5)

    @classmethod
    def create_instance_from_json(cls, json_path:str):
        with open(json_path, "r") as f:
            j = json.load(f)
            binary_location = j["web_driver"]["binary_location"]
            executable_path = j["web_driver"]["executable_path"]
            proxy = None
            try:
                proxy = j["web_driver"]["proxy"]
            except Exception:
                print("proxy is None")
            headless = False
            if j["web_driver"]["mode"] == "headless":
                headless = True
            d = CatsWebDriver(
                binary_location=binary_location,
                executable_path=executable_path,
                proxy=proxy,
                headless=headless)
        return d

    def close(self):
        """[Close WebDriverSession, if chromewebdriver dosen't kill, plsease execute "killall chromedriver"]
        
        """
        self.driver.quit()

    def reload(self):
        self.driver.refresh()

    @property
    def cookies(self):
            return self.driver.get_cookies()

    def to_request_session(self) -> CatsRequest: 
        """[summary]
        
        Returns:
            CatsRequest -- [description]
        """
        session = CatsRequest()
        for cookie in self.driver.get_cookies():
            self.driver.cookies.set(cookie["name"], cookie["value"])
        return session

    def wait_rendering_by_id(self, id, timeout=20):
        """[summary]
        
        Arguments:
            id {[type]} -- [description]
        
        Keyword Arguments:
            timeout {int} -- [description] (default: {20})
        """
        WebDriverWait(self.driver, timeout).until(EC.presence_of_element_located((By.ID, id)))

    def wait_rendering_by_class(self, _class, timeout=20):
        """[summary]
        
        Arguments:
            _class {[type]} -- [description]
        
        Keyword Arguments:
            timeout {int} -- [description] (default: {20})
        """
        WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.CLASS_NAME, _class)))

    def move(self, url):
        self.driver.get(url)

    @property
    def html(self):
        html = self.driver.page_source.encode('utf-8')
        return BeautifulSoup(html, "lxml")

    @property
    def log(self):
        result = []
        for entry in self.driver.get_log('performance'):
            result.append(entry['message'])
        return result

    def get_google_ad_iframe_ids(self):
        """[summary]
        
        Returns:
            [type] -- [description]
        """
        iframes = self.driver.html.findAll(name="iframe")
        print(f"iframe num is {len(iframes)}")
        iframe_ids = list(map(lambda x: x.get("id"), iframes))
        ad_iframe_ids = list(filter(lambda x: x != None and x.count("google_ads_iframe"), iframe_ids))
        return ad_iframe_ids
    
    def get_iframe_contents(self):
        """[summary]
        
        Returns:
            [type] -- [description]
        """
        iframe_ids = self.get_ad_iframe_ids(html)
        r = []
        for iframe_id in iframe_ids:
            iframe_elem = self.driver.find_element_by_id(iframe_id)
            self.driver.switch_to.frame(iframe_elem)
            ihtml = self.html
            r.append((iframe_id, ihtml))
            self.driver.switch_to.default_content()
        return r


import subprocess
from catscore.lib.env import get_os_type
import time

class Tor:
    @classmethod
    def wait_start(cls):
        #proc = subprocess.Popen(['tor'], stdout=subprocess.PIPE, shell=False)
        #for line in proc.stdout:
        #    print(str(line))
            #if str(line).count("100%"):
            #    proc.kill()
            #    time.sleep(3)
            #    retur
        time.sleep(1)
        return 
    
    @classmethod
    def restart(cls):
        os_type = get_os_type()
        if os_type == "linux":
            args = ['sudo', 'service', 'tor','restart']
        elif os_type == "mac": #brew services start tor
            args = ['brew', 'services', 'restart','tor']
        subprocess.call(args)
        cls.wait_start()

    @classmethod
    def start(cls):
        os_type = get_os_type()
        if os_type == "linux":
            args = ['sudo', 'service', 'tor', 'start']
        elif os_type == "mac": #brew services start tor
            args = ['brew', 'services', 'start','tor']
        subprocess.call(args)
        cls.wait_start()

    @classmethod
    def stop(cls):
        os_type = get_os_type()
        if os_type == "linux":
            args = ['sudo', 'service', 'tor','stop']
        elif os_type == "mac": #brew services start tor
            args = ['brew', 'services', 'stop','tor']
        subprocess.call(args)


from dataclasses import dataclass
import json
from pyspark import SparkContext, SQLContext
from pyspark.sql import SparkSession, DataFrame
from catscore.lib.logger import logging
import logging
import sqlalchemy
from sqlalchemy import *
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import *
import sys
from sqlalchemy_utils import database_exists, create_database, drop_database
from cats_food.libs.config import DBConfig
from peeping_cats.pclib.pc_config import PCConfig
from threading import Lock
import csv
import socket
from cats_food.libs.logger import Logging

@dataclass(frozen=True)
class MySQLConf:
    host: str
    port: str
    user: str
    pwd: str
    db_name: str

    def connection_uri(self, inteface:str):
        if inteface == "jdbc":
            uri = f"jdbc:mysql://{self.host}:{self.port}/{self.db_name}?user={self.user}&password={self.pwd}"
        elif inteface == "pymysql":
            uri =  'mysql+pymysql://%s:%s@%s/%s?charset=utf8' % (self.user, self.pwd, self.host + ":" + self.port, self.db_name)
        print(uri)
        return uri

    @classmethod
    def from_json(cls, path):
        with open(path, "r") as f:
            j = json.load(f)
            m = MySQLConf(
                host=j["mysql"]["host"],
                port=j["mysql"]["port"],
                user=j["mysql"]["user"],
                pwd=j["mysql"]["pwd"],
                db_name=j["mysql"]["db_name"])
        return m

class MySQLSpark:
    @classmethod
    def read(cls, spark:SparkSession, mysql_conf:MySQLConf, table_name:str) -> DataFrame:
        sql_context = SQLContext(spark.sparkContext)
        return sql_context.read.format("jdbc").options(url=mysql_conf.connection_uri("jdbc"), dbtable=table_name).load()
    
    @classmethod
    def overwrite(cls, spark:SparkSession, table_name: str, df:DataFrame, mysql_conf:MySQLConf):
        df.write.jdbc(mysql_conf.connection_uri("jdbc"), table=table_name, mode='overwrite')


class MySQLAlchemySingleton:
    Base = declarative_base()
    DB_NAME = "peeping_cats"
    """
    
    """
    _unique_instance = None
    _lock = Lock()  # Class Lock
    session = None

    def __new__(cls):
        raise NotImplementedError('Cannot initialize via Constructor')

    @classmethod
    def __internal_new__(cls):
        return super().__new__(cls)

    @classmethod
    def initialize(cls, mysql_conf: MySQLConf, echo=False):
        """

        :param config:
        :return:
        """
        logging.info(f"db config {mysql_conf}")
        db_setting = mysql_conf.connection_uri(inteface="pymysql")
        logging.info("db_setting : {}".format(db_setting))
        print("db_setting : {}".format(db_setting))

        cls.engine = create_engine(
            db_setting,
            encoding="utf-8",
            echo=echo  # Trueだと実行のたびにSQLが出力される
        )
        # Sessionの作成
        cls.session = scoped_session(
            sessionmaker(
                autocommit=False,
                autoflush=False,
                bind=cls.engine
            )
        )
        cls.query = cls.session.query_property()

        if not cls._unique_instance:
            with cls._lock:
                if not cls._unique_instance:
                    cls._unique_instance = cls.__internal_new__()
                    logging.info("PCSQLAlchemySingleton instance created.")
        else:
            message = "PCSQLAlchemySingleton instance has already created."
            logging.error(message)
            raise RuntimeError(message)

    @classmethod
    def close(cls):
        logging.info("PCSQLAlchemySingleton close")
        cls.session.close()
        cls._unique_instance = None

    @classmethod
    def get_instance(cls):
        """
        Get instance of PCSQLAlchemySingleton
        :return:
        """
        logging.info(f"get_instance : {cls._unique_instance}")
        return cls._unique_instance

    @classmethod
    def create_db(cls):
        """
        create database
        Attention!! create_dbを呼ぶファイルに対象modelがimportされていないと、tableが作成されない
        :return:
        """
        if not database_exists(cls.engine.url):
            logging.info("create database")
            create_database(cls.engine.url)

        cls.Base.metadata.create_all(bind=cls.engine)

        logging.info("db created.")

    @classmethod
    def drop_db(cls):
        if database_exists(cls.engine.url):
            logging.info(f"drop database {cls.engine.url}")
            #drop_database(cls.engine.url)
            cls.Base.metadata.drop_all(bind=cls.engine)
        else:
            logging.info("db not found")

    @classmethod
    def insert(cls, il, allow_duplicate = False):
        """

        :param il:
        :return:
        """
        # add list, with remove None
        if allow_duplicate:
            for item in il:
                try:
                    cls.session.add(item)
                    cls.session.commit()
                except:
                    logging.debug(f"{sys.exc_info()}")
                    logging.debug(f"key is duplicate]: {item}")
                    cls.session.rollback()
        else:
            try:
                cls.session.add_all(filter(None, il))
                cls.session.commit()
            except:
                Logging.error("sys.exc_info : {}".format(sys.exc_info()))
                Logging.error("session rollback")
                cls.session.rollback()
                raise RuntimeError("sys.exc_info : {}".format(sys.exc_info()))
    @classmethod
    def tables_to_csv(cls, tables):
        for table in tables:
            filename = f'{PCConfig.dump_path}/{table.__tablename__}_{socket.gethostname()}.csv'
            logging.info(f"{table} output to {filename}")
            with open(filename, 'w') as outfile:
                outcsv = csv.writer(outfile)
                cursor = MySQLAlchemySingleton.get_instance().session.query(table)
                header = table.__table__.columns.keys()
                logging.info(f"tables_to_csv header: {header}")
                # dump column titles (optional)
                outcsv.writerow(header)
                for cur in cursor.yield_per(10):
                    r = [getattr(cur, c) for c in header]
                    remove_n = list(map(lambda x: str(x).replace("\n",""), r))
                    outcsv.writerow(remove_n)
                # dump rows => fetchallだとmemoryが足りない
                #outcsv.writerows(cursor.fetchall())
                #print(filename)


import unittest
from catscore.http.request import CatsRequest
from catscore.http.tor import Tor

class TestCatsRequest(unittest.TestCase):
    def test_get(self):
        request:CatsRequest = CatsRequest()
        responseHtml = request.get(url="https://yahoo.co.jp", response_content_type="html")
        responseJson = request.get(url="https://api.rescala.jp/current_stock_info", response_content_type="json")
        print(responseJson)
        request.close()
        
    def tor(self):
        """[python -m unittest tests.http.request.TestCatsRequest.tor
        https://lets-hack.tech/programming/languages/python/tor/
        must execute pip install pysocks
        # Install for mac
        brew install tor
        ## start 
        brew services start tor
        ## stop
        brew services stop tor
        ]
        """

        proxy = None
        request:CatsRequest = CatsRequest(proxy=proxy)
        print(f"without tor: {request.get_global_info()['ip']}")
        request.close()
        
        proxy = {'http':'socks5://127.0.0.1:9050','https':'socks5://127.0.0.1:9050'}
        request:CatsRequest = CatsRequest(proxy=proxy)
        print(f"with tor: {request.get_global_info()['ip']}")
        print(f"with tor: {request.get_global_info()['ip']}")
        request.close()
        
        print(f"tor restart")
        Tor.restart()
        request:CatsRequest = CatsRequest(proxy=proxy)
        print(f"with tor: {request.get_global_info()['ip']}")
        print(f"with tor: {request.get_global_info()['ip']}")
        request.close()

if __name__ == "__main__":
    unittest.main()


import os
from catscore.lib.time import get_today_date, get_current_time
from catscore.lib.logger import CatsLogging as logging
import pathlib
import shutil

def daily_worker_out(output_folder_path: str, s: str, function_name: str, extension: str, do_zip:bool=True):
    # output init
    function_folder_path = f"{output_folder_path}/{function_name}"
    daily_folder_path = f"{function_folder_path}/{get_today_date()}"
    #zip_daily_folder_path = daily_folder_path + ".zip"
    worker_out_path = daily_folder_path + f"/{function_name}_{get_current_time()}.{extension}"
    logging.info(f"daily_folder_path: {daily_folder_path}")
    logging.info(f"worker_out_path: {worker_out_path}")
    
    # check daily_folder
    daily_folder = pathlib.Path(daily_folder_path)
    if not daily_folder.exists():
        logging.info(f"daily_folder_path: mkdir {daily_folder_path}")
        daily_folder.mkdir(parents=True)
        
    # output str
    with open(worker_out_path, 'w') as f:
        f.write(s)
        
    # zip daily_folder check&create
    #if do_zip:
    #    files = sorted(os.listdir(function_folder_path))
    #    latest_timestamp = files[-1]
    #    #zip_files = sorted(filter(lambda f: "zip" in f, os.listdir(function_folder_path)))
    #    if get_today_date() > latest_timestamp:
    #        logging.info("daily_worker_out: zipping")
    #        shutil.make_archive(daily_folder_path, 'zip', root_dir=daily_folder_path)